@@ -2,13 +2,16 @@ package App::ForkProve;
 
 use strict;
 use 5.008_001;
-our $VERSION = '0.2.3';
+our $VERSION = '0.3.0';
 
 use App::Prove;
 use Getopt::Long ':config' => 'pass_through';
 
 use App::ForkProve::SourceHandler;
 
+our @Blacklists = qw( Test::SharedFork );
+our @Captured;
+
 sub run {
     my($class, @args) = @_;
 
@@ -40,16 +43,7 @@ sub run {
 
     for (@modules) {
         my($module, @import) = split /[=,]/;
-
-        my $warn = sub {
-            if ($_[1] eq 'Test/Builder.pm') {
-                require Carp;
-                Carp::cluck("Loading $module ended up requiring Test::Builder, " .
-                  "which is known to cause issues with forkprove.\n");
-            }
-            return;
-        };
-        local @INC = ($warn, @inc, @INC);
+        local @INC = ($class->blacklist_sub, @inc, @INC);
 
         eval "require $module" or die $@;
         $module->import(@import);
@@ -60,6 +54,39 @@ sub run {
     $app->run;
 }
 
+sub file_to_pkg {
+    local $_ = shift;
+    s|/|::|g;
+    s|\.pm$||;
+    $_;
+}
+
+sub pkg_to_file {
+    local $_ = shift;
+    s|::|/|g;
+    "$_.pm";
+}
+
+sub blacklist_sub {
+    my $class = shift;
+
+    my %blacklisted = map { $_ => 1 } @Blacklists;
+
+    return sub {
+        my($coderef, $filename) = @_;
+
+        my $package = file_to_pkg($filename);
+        if ($blacklisted{$package}) {
+            my @content = ("package $package; push \@App::ForkProve::Captured, __PACKAGE__; 1;");
+            return sub {
+                return ($_ = shift @content) ? 1 : 0;
+            };
+        }
+
+        return;
+    };
+}
+
 1;
 __END__
 
