@@ -67,6 +67,19 @@ sub _run {
     # note that this has to be dynamically scoped and can't go to other subs
     "" =~ /^/;
 
+    # Test::Builder is loaded? Reset the $Test object to make it unaware
+    # that it's a forked off proecess so that subtests won't run
+    if (defined $Test::Builder::Test) {
+        $Test::Builder::Test->reset;
+    }
+
+    # Eliminated blacklisted modules from %INC so that child can reload on demand
+    for my $package (@App::ForkProve::Captured) {
+        no strict 'refs';
+        undef %{"$package\::"};
+        delete $INC{ App::ForkProve::pkg_to_file($package) };
+    }
+
     # do() can't tell if a test can't be read or a .t's last statement
     # returned undef with $! set somewhere. Fortunately in case of
     # prove, non-readable .t will fail earlier in prove itself.
